// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// The Initial Developer of the Original Code is Aseem Kishore.                       //
// Portions created by Scott Williamson are Copyright (c) 2006.  All Rights Reserved. //
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

// -- DOMAIN FILTERING -- //

function thumbstrips_trimDomain(domain) {
	// take out the "www." if it exists in the beginning
	if (domain.substr(0, 4) === "www.") {
		domain = domain.substr(4);
	}
	
	return domain;
}

function thumbstrips_fillDomains(button) {
	// remove the old menu
	
	for (var a = 0; a < button.childNodes.length; a++) {
		button.removeChild(button.childNodes[a]);
		a--;
	}
	
	var menu = document.createElement("menupopup");
	
	// cycle through each filmstrip node and put in its domain -- don't repeat
	
	var nodes = thumbstrips_filmstrip.getNodes();
	var menuDomains = new Array();
	var i;
	var domain;
	
	for (i = 0; i < nodes.length; i++) {
		domain = thumbstrips_trimDomain(nodes[i].domain);
		
		// use a hash map to preven duplicates, and it's also ordered alphabetically
		menuDomains[domain] = domain;
	}
	
	// add an all item and a separator
	
	var all = document.createElement("menuitem");
	var sep = document.createElement("menuseparator");
	
	all.setAttribute("label", thumbstrips_contentAllLabel);
	all.setAttribute("type", "checkbox");
	
	if (thumbstrips_domainFiltering === false) {
		all.setAttribute("checked", "true");
	} else {
		all.setAttribute("checked", "false");
	}
	
	// important -- don't use autocheck because this needs to read/set manually
	// important -- don't do event.preventBubble() since the button's oncommand
	// is to filter the domains after any change.
	all.setAttribute("autocheck", "false");
	all.setAttribute("oncommand", "thumbstrips_selectAllDomains(this);");
	
	menu.appendChild(all);
	menu.appendChild(sep);
	
	// then cycle through each domain in the hash map and append it to the menu
	// but first copy it over to maintain alphabetical order
	
	var copyMenuDomains = thumbstrips_propCopy(menuDomains).sort();
	
	for (i = 0; i < copyMenuDomains.length; i++) {
		// if filtering is off, add it to the menu as checked, and don't bother
		// adding it to the currentDomains array
		
		var item = document.createElement("menuitem");
		
		item.setAttribute("label", copyMenuDomains[i]);
		item.setAttribute("type", "checkbox");
		
		if (thumbstrips_domainFiltering === false) {
			item.setAttribute("checked", "false");
		} else {
			if (thumbstrips_currentDomains[copyMenuDomains[i]] === true) {
				item.setAttribute("checked", "true");
			} else {
				item.setAttribute("checked", "false");
			}
		}
		
		// important -- don't have autocheck on because check needs to be read/set manually
		// important -- don't do event.preventBubble() since the button's oncommand
		// is to filter the domains after any change.
		item.setAttribute("autocheck", "false");
		item.setAttribute("oncommand", "thumbstrips_toggleDomain(this);");
		
		menu.appendChild(item);
	}
	
	button.appendChild(menu);
}


function thumbstrips_filterDomainsOnOneNode(node, domain) {
	domain = thumbstrips_trimDomain(domain);
	
	if (thumbstrips_domainFiltering === false) {
		thumbstrips_updateHiddenStatus(node, "domain", false);
	} else {
		if (thumbstrips_currentDomains[domain] === true) {
			thumbstrips_updateHiddenStatus(node, "domain", false);
		} else {
			thumbstrips_updateHiddenStatus(node, "domain", true);
		}
	}
}


function thumbstrips_filterDomainsOnAllNodes() {
	var nodes = thumbstrips_box.childNodes;
	
	for (var i = 0; i < nodes.length; i++) {
		var vc = nodes[i].getAttribute("vc");
		var id = vc ? thumbstrips_getOriginalId(nodes[i].id, vc) : nodes[i].id;
		
		var domain = thumbstrips_filmstrip.getNodeById(id).domain;
		
//		// take out the "www." if it exists in the beginning
//		if (domain.substr(0, 4) === "www.") {
//			domain = domain.substr(4);
//		}
//		
//		if (thumbstrips_domainFiltering === false) {
////			nodes[i].removeAttribute("hidden");
//			thumbstrips_updateHiddenStatus(nodes[i], "domain", false);
//		} else {
//			if (thumbstrips_currentDomains[domain] === true) {
////				nodes[i].removeAttribute("hidden");
//				thumbstrips_updateHiddenStatus(nodes[i], "domain", false);
//			} else {
////				nodes[i].setAttribute("hidden", "true");
//				thumbstrips_updateHiddenStatus(nodes[i], "domain", true);
//			}
//		}

		thumbstrips_filterDomainsOnOneNode(nodes[i], domain);
	}
}


function thumbstrips_selectAllDomains(item) {
	if (thumbstrips_domainFiltering === false) {
		return;
	}
	
	thumbstrips_domainFiltering = false;
	thumbstrips_currentDomains = new Array();
	
	var menu = item.parentNode;
	var button = menu.parentNode;
	
	button.setAttribute("label", thumbstrips_contentButtonAllLabel);
	item.setAttribute("checked", "true");
	
	// begin after the all and separator items and uncheck everything
	for (var i = 2; i < menu.childNodes.length; i++) {
		menu.childNodes[i].setAttribute("checked", "false");
	}
}


function thumbstrips_toggleDomain(item) {
	var menu = item.parentNode;
	var button = menu.parentNode;
	
	if (thumbstrips_domainFiltering === false) {
		thumbstrips_domainFiltering = true;
		
		var all = menu.childNodes[0];
		
		all.setAttribute("checked", "true");
		button.setAttribute("label", thumbstrips_contentButtonFilteredLabel);
	}
	
	var domain = item.getAttribute("label");
	
	// add/remove it from currentDomains, but don't remove it if it's the only one
	
	if (item.getAttribute("checked") === "true") {
		// assert currentDomains exists and currentDomains[domain] exists and = true
		
		if (thumbstrips_propLength(thumbstrips_currentDomains) <= 1) {
			return;
		}
		
		delete thumbstrips_currentDomains[domain];
		item.setAttribute("checked", "false");
	} else {
		// assert currentDomains exists and currentDomains[domain] doesn't exist or exists and = false
		
		thumbstrips_currentDomains[domain] = true;
		item.setAttribute("checked", "true");
	}
}