// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// The Initial Developer of the Original Code is Aseem Kishore.                       //
// Portions created by Scott Williamson are Copyright (c) 2006.  All Rights Reserved. //
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

// -- DRAWING -- //


function thumbstrips_openTray() {
	if (thumbstrips_tray.getAttribute("hidden")) {	// if tray is closed
		thumbstrips_drawFilmstrip(); // optional, but opening/closing is a good way to "fix" bugs
		
		thumbstrips_tray.removeAttribute("hidden");
		thumbstrips_splitter.removeAttribute("hidden");
		
		var item = document.getElementById("thumbstrips_viewFilmstripMenuItem");
		item.setAttribute("checked", "true");
		thumbstrips_button.setAttribute("checked", "true");
		
		if (thumbstrips_prefs.getBoolPref("options.mouse_scroll") === true) {
			window.addEventListener("mousemove", thumbstrips_updateMousePosition, true);
			thumbstrips_mouseScrollUpdater = window.setInterval(thumbstrips_mouseScroll, thumbstrips_prefs.getIntPref("time.mouse_update_frequency"));
		}
		
		thumbstrips_splitter.addEventListener("mousedown", thumbstrips_beginResizingThumbnails, false);
		thumbstrips_splitter.addEventListener("mouseup", thumbstrips_endResizingThumbnails, false);
		
		// if the current thumbnail is visible, scroll to the new thumbnail
		/*if (thumbstrips_currentNode && thumbstrips_currentNode.id) {
			var container = document.getElementById(thumbstrips_currentNode.id);
			if (container && !container.getAttribute("hidden")) {
				thumbstrips_scrollbox.ensureElementIsVisible(container);
			}
		}*/
		// the above commented code has been moved to:
		thumbstrips_skipToCurrent();
		
	} else {	// if tray is open
		thumbstrips_tray.setAttribute("hidden", "true");
		thumbstrips_splitter.setAttribute("hidden", "true");
		
		var item = document.getElementById("thumbstrips_viewFilmstripMenuItem");
		item.setAttribute("checked", "false");
		thumbstrips_button.setAttribute("checked", "false");
	
		window.removeEventListener("mousemove", thumbstrips_updateMousePosition, true);
		window.clearInterval(thumbstrips_mouseScrollUpdater);
		
		thumbstrips_splitter.removeEventListener("mousedown", thumbstrips_beginResizingThumbnails, false);
		thumbstrips_splitter.removeEventListener("mouseup", thumbstrips_endResizingThumbnails, false);
	}
}





function thumbstrips_drawFilmstrip() {
	var nodes = thumbstrips_filmstrip.getNodes();
	var i;
	
	// First, remove all children
	
	for (i = 0; i < thumbstrips_box.childNodes.length; i = 0) {
		thumbstrips_box.removeChild(thumbstrips_box.childNodes[0]);
	}
	
	// If our current sorting is to show every visit, we need to keep track of
	// the various instances of each URL (visit counts)
	var visitCounts = new Array();
	
	// Then add each node in a container
	
	for (i = 0; i < nodes.length; i++) {
		var node = nodes[i];
		
//		var vcout = "visitCounts, node.id = " + node.id + "\n";
//		vcout += "pre: visitCounts[node.id] = " + visitCounts[node.id] + "\n";
		
		if (visitCounts[node.id] === undefined) {
			visitCounts[node.id] = 0;
//			vcout += "undefined --> making 0\n"
		} else {
			visitCounts[node.id]++;
//			vcout += "defined --> incrementing\n";
		}
		
//		vcout += "post: visitCounts[node.id] = " + visitCounts[node.id] + "\n";
//		thumbstrips_log(vcout);
		
		var vc = visitCounts[node.id];
		var id;
		var idvcFlag = false;
		
		if (thumbstrips_prefs.getBoolPref("sorting.every_visit") === true) {
			if (thumbstrips_currentNode && thumbstrips_currentNode.id === node.id && vc === (node.getVisitCount() - 1)) {
				id = node.id;
			} else {
				id = node.id + vc;
				idvcFlag = true;
			}
		} else {
			id = node.id;
		}
		
//		var output = "drawing!\n";
//		output += "i = " + i + ", node.id = " + node.id + "\n";
//		output += "vc = " + vc + ", id = " + id + ", idvcFlag = " + idvcFlag + "\n";
//		thumbstrips_log(output);
		
		var container = document.createElement("vbox");
		var padding = document.createElement("label");
		var title = document.createElement("label");
		var canvas = null;
		var time = document.createElement("label");
		var popup = document.createElement("popup");
		var pDelete = document.createElement("menuitem");
		var pSep = document.createElement("menuseparator");
		var pProps = document.createElement("menuitem");
		
		if (thumbstrips_prefs.getBoolPref("sorting.every_visit") === true) {
			if (vc === (node.getVisitCount() - 1)) {
				canvas = node.thumbnail;
			} else {
				canvas = thumbstrips_newCanvas(id);
				thumbstrips_copyCanvas(node.thumbnail, canvas);
			}
		} else {
			// use the node's thumbnail directly
			canvas = node.thumbnail;
		}
		
		container.setAttribute("class", thumbstrips_containerClass);
		
		if (thumbstrips_currentNode && thumbstrips_currentNode.id === node.id) {
			if (thumbstrips_prefs.getBoolPref("sorting.every_visit") === true) {
				if (vc === (node.getVisitCount() - 1)) {
					container.setAttribute("class", thumbstrips_containerClass + " " + thumbstrips_currentClass);
				}
			} else {
				container.setAttribute("class", thumbstrips_containerClass + " " + thumbstrips_currentClass);
			}
		}
		
		container.setAttribute("id", id);
		container.setAttribute("oncommand", "thumbstrips_goToNode(this, event);");
		container.setAttribute("onclick", "thumbstrips_checkForMiddleClick(this, event);");
//		container.setAttribute("onclick", "thumbstrips_handleNodeClick(this, event);");
		container.setAttribute("onmouseover", "thumbstrips_showNodeInfo(\"" + id + "\");");
		container.setAttribute("onmouseout", "thumbstrips_clearNodeInfo(\"" + id + "\");");
		container.setAttribute("align", "center");
		
		if (idvcFlag === true) {
			container.setAttribute("vc", vc);
		}
		
		padding.setAttribute("class", thumbstrips_paddingClass);
		padding.setAttribute("id", thumbstrips_paddingClass + id);
		padding.setAttribute("value", " ");
		
		title.setAttribute("class", thumbstrips_titleClass);
		title.setAttribute("id", thumbstrips_titleClass + id);
		title.setAttribute("value", node.title);
		title.setAttribute("crop", "end");
//		title.setAttribute("hidden", "true");
		
		// canvas is already initialized in thumbstrips_init(node)
		
		time.setAttribute("class", thumbstrips_timeClass);
		time.setAttribute("id", thumbstrips_timeClass + id);
		
		thumbstrips_updateTimeForOneNode(time, node, vc);
		
		time.setAttribute("crop", "end");
		
		popup.setAttribute("class", thumbstrips_popupClass);
		popup.setAttribute("id", thumbstrips_popupClass + id);
		popup.setAttribute("onclick", "event.preventBubble();");
		popup.appendChild(pDelete);
		popup.appendChild(pSep);
		popup.appendChild(pProps);
		
		pDelete.setAttribute("label", thumbstrips_deleteLabel);
		pDelete.setAttribute("tooltiptext", thumbstrips_deleteTooltiptext);
		
		if (idvcFlag === true) {
			pDelete.setAttribute("oncommand", "thumbstrips_removeThumbnail(\"" + id + "\", \"" + vc + "\"); event.preventBubble();");
		} else {
			pDelete.setAttribute("oncommand", "thumbstrips_removeThumbnail(\"" + id + "\"); event.preventBubble();");
		}
		
		pProps.setAttribute("label", thumbstrips_propsLabel);
		pProps.setAttribute("tooltiptext", thumbstrips_propsTooltiptext);
		
		if (idvcFlag === true) {
			pProps.setAttribute("oncommand", "thumbstrips_showProperties(\"" + id + "\", \"" + vc + "\"); event.preventBubble();");
		} else {
			pProps.setAttribute("oncommand", "thumbstrips_showProperties(\"" + id + "\"); event.preventBubble();");
		}
		
//		container.appendChild(padding);
		container.appendChild(title);
		container.appendChild(canvas);
		container.appendChild(time);
		container.appendChild(popup);
		container.setAttribute("context", popup.id);
		
		// if the canvas is hidden, the page hasn't loaded yet, hide the container
		if (canvas.getAttribute("hidden")) {
			thumbstrips_updateHiddenStatus(container, "load", true);
		}
		
		// if this domain is being filtered, hide the container
		thumbstrips_filterDomainsOnOneNode(container, node.domain);
		
		// if this domain doesn't match the search, hide it
//		thumbstrips_searchOneNode(nodes[i].id);
		thumbstrips_searchOneNode(container);
		
//		// take out the "www." if it exists in the beginning
//		if (domain.substr(0, 4) === "www.") {
//			domain = domain.substr(4);
//		}
//		
//		if (thumbstrips_domainFiltering === false) {
////			container.removeAttribute("hidden");
//			thumbstrips_updateHiddenStatus(container, "domain", false);
//		} else {
//			if (thumbstrips_currentDomains[domain] === true) {
////				container.removeAttribute("hidden");
//				thumbstrips_updateHiddenStatus(container, "domain", false);
//			} else {
////				container.setAttribute("hidden", "true");
//				thumbstrips_updateHiddenStatus(container, "domain", true);
//			}
//		}

		thumbstrips_box.appendChild(container);
	}
}


function thumbstrips_getOriginalId(id, visitCount, debug) {
//	var a = visitCount ? id.slice(0, id.lastIndexOf(visitCount.toString())) : id;
//	
//	thumbstrips_log("f_gO, id = " + id + ", vc = " + visitCount + "\n          =======> " + a);
//	
//	return a;

//	var a = visitCount ? visitCount.toString().length : 0;
//	var b = id.slice(0, id.length - a);
//	
//	if (debug) {
//		thumbstrips_log("f_gO, id = " + id + ", vc = " + visitCount + "\n          =======> " + b + "\n(a = " + a + ")");
//	}
	
	return visitCount ? id.slice(0, id.lastIndexOf(visitCount)) : id;
}



function thumbstrips_removeThumbnail(id, visitCount) {
	var origId = thumbstrips_getOriginalId(id, visitCount, true);
	var node = thumbstrips_filmstrip.getNodeById(origId);
	
//	thumbstrips_log("removing, id = " + id + ", vC = " + visitCount + ", origId = " + origId);
//	thumbstrips_log("node = " + node);
	
	thumbstrips_filmstrip.removeNode(node, parseInt(visitCount));
	
	if (thumbstrips_currentNode && node.id === thumbstrips_currentNode.id) {
		thumbstrips_clearUpdaters();
		thumbstrips_deletedUrl = thumbstrips_currentUrl;
	}
	
	thumbstrips_drawFilmstrip();
}


function thumbstrips_showProperties(id, visitCount) {
	var origId = thumbstrips_getOriginalId(id, visitCount, true);
	var node = thumbstrips_filmstrip.getNodeById(origId);
	
	window.openDialog("chrome://thumbstrips/content/propsdialog.xul", "", "chrome,modal,centerscreen", node);
}



function thumbstrips_updateHiddenStatus(node, type, hidden) {
	// there will be multiple forces that determine whether a node is hidden.
	// for example:
	
	// 1. the page hasn't loaded yet, keep it hidden at first
	// 2. the page's domain is being filtered out
	// 3. the search box has text in it that doesn't match the page
	
	// so, each node will have an array of boolean hiddens, and it takes just
	// one index to be set to true for the node to be hidden (an OR problem).
	
	// this method takes a node, a type (e.g. "domain", "load", etc. -- any type
	// of string that describes the force) and a boolean and updates the node's
	// array of hidden booleans. it then cycles through each index in the array
	// and updates the node's hidden status if any are set to true.
	
	if (!node.hiddenArray) {
		// create an array if none exists yet
		node.hiddenArray = new Array();
	}
	
	// update the specific type of hidden force
	node.hiddenArray[type] = hidden;
	
	// cycle through and set the hidden status accordingly
	for (var i in node.hiddenArray) {
		if (node.hiddenArray[i] === true) {
			node.setAttribute("hidden", "true");
			return;
		}
	}
	
	node.removeAttribute("hidden");
}




// time is a XUL label element
// node is a WebpageNode
function thumbstrips_updateTimeForOneNode(time, node, visitCount) {
	if (thumbstrips_currentNode && thumbstrips_currentNode.id === node.id && time.id === (thumbstrips_timeClass + node.id)) {
		time.setAttribute("value", thumbstrips_nowLabel);
	} else {
		if (thumbstrips_prefs.getBoolPref("sorting.first_visit") === true) {
			time.setAttribute("value", thumbstrips_getTimeAgo(node.getVisitList()[0]));
		} else if (thumbstrips_prefs.getBoolPref("sorting.last_visit") === true) {
			var v = node.getVisitList().length - 1;
			time.setAttribute("value", thumbstrips_getTimeAgo(node.getVisitList()[v]));
		} else {
			time.setAttribute("value", thumbstrips_getTimeAgo(node.getVisitList()[visitCount]));
		}
	}
}

function thumbstrips_updateTimes() {
	var nodes = thumbstrips_box.childNodes;
	
	for (var i = 0; i < nodes.length; i++) {
		var vc = nodes[i].getAttribute("vc");
		var id = vc ? thumbstrips_getOriginalId(nodes[i].id, vc) : nodes[i].id;
		
		var time = document.getElementById(thumbstrips_timeClass + nodes[i].id);
		var node = thumbstrips_filmstrip.getNodeById(id);
		
		if (!time) {
			thumbstrips_log("no time! id = " + nodes[i].id + ", original = " + id);
		}
		
		thumbstrips_updateTimeForOneNode(time, node, vc);
	}
}




function thumbstrips_beginResizingThumbnails() {
//	thumbstrips_resizeUpdater = window.setInterval(thumbstrips_resizeThumbnails, 20);
	window.addEventListener("mousemove", thumbstrips_resizeThumbnails, true);
}

function thumbstrips_resizeThumbnails() {
//	thumbstrips_log("resizing...");
	
	var nodes = thumbstrips_box.childNodes;
	
	if (nodes.length === 0) {
		return;
	}
	
	var hBox;
	
	if (thumbstrips_hidden.getAttribute("hidden")) {	// i.e. there is a scrollbar
		hBox = thumbstrips_box.boxObject.height - 16;
	} else {	// i.e. there is no scrollbar
		hBox = thumbstrips_box.boxObject.height;
	}
	
//	var hBox = thumbstrips_box.boxObject.height;
	var hMin = thumbstrips_prefs.getIntPref("style.thumb_min_height");
	var hMax = thumbstrips_prefs.getIntPref("style.thumb_max_height");
	var hText = thumbstrips_textHeight;
	var hNew = ((hBox - hText) < hMin) ? hMin : (((hBox - hText) > hMax) ? hMax : (hBox - hText));
	
	var tempThumb = document.getElementById(thumbstrips_thumbnailClass + nodes[0].id);
		
	// in some cases, node.id will be [url][vc] while canvas.id will be [canvas-class][url]
	if (!tempThumb && nodes[0].getAttribute("vc")) {
		var tempId = thumbstrips_getOriginalId(nodes[0].id, nodes[0].getAttribute("vc"));
		tempThumb = document.getElementById(thumbstrips_thumbnailClass + tempId);
	}
	
	var scale = hNew / tempThumb.displayHeight;
	
	for (var i = 0; i < nodes.length; i++) {
//		var thumb = thumbstrips_filmstrip.getNodeById(nodes[i].id).thumbnail;
		var thumb = document.getElementById(thumbstrips_thumbnailClass + nodes[i].id);
		
		if (!thumb && nodes[i].getAttribute("vc")) {
			var tempId = thumbstrips_getOriginalId(nodes[i].id, nodes[i].getAttribute("vc"));
			thumb = document.getElementById(thumbstrips_thumbnailClass + tempId);
		}
		
		thumb.displayHeight = hNew;
		thumb.displayWidth *= scale;
		
		thumb.style.height = thumb.displayHeight + "px";
		thumb.style.minHeight = thumb.style.height;
		thumb.style.maxHeight = thumb.style.height;
		
		thumb.style.width = thumb.displayWidth + "px";
		thumb.style.minWidth = thumb.style.width;
		thumb.style.maxWidth = thumb.style.width;
	}
	
	thumbstrips_currentHeight = hNew;
}

function thumbstrips_endResizingThumbnails() {
//	window.clearInterval(thumbstrips_resizeUpdater);
	window.removeEventListener("mousemove", thumbstrips_resizeThumbnails, true);
	thumbstrips_resizeThumbnails();
}

function thumbstrips_addScrollbar() {
	var inner = document.getAnonymousNodes(thumbstrips_box).item(0);
	
	if (thumbstrips_box.boxObject.width >= inner.boxObject.width) {
		return;
	}
	
	var h = thumbstrips_box.boxObject.height;
	
	thumbstrips_hidden.setAttribute("hidden", "true");
//	thumbstrips_box.setAttribute("style", "height: " + (h + 16) + "px; overflow: auto;");
	thumbstrips_box.style.height = (h + 16) + "px";
	thumbstrips_box.style.overflow = "auto";
	
//	thumbstrips_box.setAttribute("onmousedown", "thumbstrips_considerStopMouseScroll(event);");
//	thumbstrips_box.setAttribute("onmouseup", "thumbstrips_considerStartMouseScroll(event);");
	
//	thumbstrips_log("added scrollbar");
}

function thumbstrips_removeScrollbar() {
	var inner = document.getAnonymousNodes(thumbstrips_box).item(0);
	
	if (thumbstrips_box.boxObject.width <= inner.boxObject.width) {
		return;
	}
	
	var h = thumbstrips_box.boxObject.height;
	
//	thumbstrips_box.setAttribute("style", "height: " + (h - 16) + "px;");
	thumbstrips_box.style.height = (h - 16) + "px";
	thumbstrips_box.style.removeProperty("overflow");
	thumbstrips_hidden.removeAttribute("hidden");
	
//	thumbstrips_box.removeAttribute("onmousedown");
//	thumbstrips_box.removeAttribute("onmouseup");
	
//	thumbstrips_log("removed scrollbar");
}

function thumbstrips_changeScrollbar() {
	var inner = document.getAnonymousNodes(thumbstrips_box).item(0);
	
	if (thumbstrips_box.boxObject.width < inner.boxObject.width) { // overflow
//		thumbstrips_log("overflow happened");
	} else {
//		thumbstrips_log("underflow happened");
	}
}


function thumbstrips_considerStopMouseScroll(event) {
	if (thumbstrips_prefs.getBoolPref("options.mouse_scroll") === true) {
		var mY = event.screenY;
		var sY = thumbstrips_box.screenY + thumbstrips_box.boxObject.height - 16;
		
		if (mY >= sY) { // we clicked on the scrollbar or lower
			thumbstrips_log("we clicked on the scrollbar or lower");
			window.removeEventListener("mousemove", thumbstrips_updateMousePosition, true);
			window.clearInterval(thumbstrips_mouseScrollUpdater);
			thumbstrips_mousePause = true;	
		}
	}
}

function thumbstrips_considerStartMouseScroll(event) {
	if (thumbstrips_prefs.getBoolPref("options.mouse_scroll") === true && thumbstrips_mousePause === true) {
		window.addEventListener("mousemove", thumbstrips_updateMousePosition, true);
		thumbstrips_mouseScrollUpdater = window.setInterval(thumbstrips_mouseScroll, thumbstrips_prefs.getIntPref("time.mouse_update_frequency"));
		thumbstrips_mousePause = false;
	}
}



function thumbstrips_updateMousePosition(event) {
	thumbstrips_mouseX = event.screenX;
	thumbstrips_mouseY = event.screenY;
}





function thumbstrips_toggleSortingPreferences(menu) {
	var firstVisit = menu.childNodes[0].getAttribute("checked");
	var lastVisit = menu.childNodes[1].getAttribute("checked");
	var everyVisit = menu.childNodes[2].getAttribute("checked");
	
	var values = [ false, false, false ];
	
	if (firstVisit === "true") {
//		thumbstrips_log("sorting by first visit");
		values[0] = true;
	} else if (lastVisit === "true") {
//		thumbstrips_log("sorting by last visit");
		values[1] = true;
	} else {
//		thumbstrips_log("sorting by every visit");
		values[2] = true;
	}
	
	thumbstrips_prefs.setBoolPref("sorting.first_visit", values[0]);
	thumbstrips_prefs.setBoolPref("sorting.last_visit", values[1]);
	thumbstrips_prefs.setBoolPref("sorting.every_visit", values[2]);
}


function thumbstrips_setSorting() {
	
	if (thumbstrips_prefs.getBoolPref("sorting.first_visit") === true) {
		thumbstrips_filmstrip.sortByFirstVisit();
	} else if (thumbstrips_prefs.getBoolPref("sorting.last_visit") === true) {
		thumbstrips_filmstrip.sortByLastVisit();
	} else {
		thumbstrips_filmstrip.sortByEveryVisit();
	}
	
	thumbstrips_drawFilmstrip();
	
}