// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// The Initial Developer of the Original Code is Aseem Kishore.                       //
// Portions created by Scott Williamson are Copyright (c) 2006.  All Rights Reserved. //
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

// -- REGULAR PAGE UPDATING -- //

function thumbstrips_resetUpdaters() {
	if (thumbstrips_recording === false) {
		var temp = thumbstrips_filmstrip.getNodeById(content.location.toString());
		
		if (temp) {
			thumbstrips_currentNode = temp;
		} else {
			thumbstrips_currentNode = null;
		}
		
		thumbstrips_drawFilmstrip();
		
		return;
	}
	
	var url = content.location.toString();
	var protocol = content.location.protocol.toString();
	var host = "";
	
	try {
		host = content.location.host;
	} catch (e) {
		host = "";
	}
	
//	thumbstrips_log("RESETTING, url = " + url + ", current = " + thumbstrips_currentUrl);
	
	// Loading a URL sometimes fires a location change event twice 
	if (url === thumbstrips_currentUrl) {
		return;
	}
	
	thumbstrips_currentUrl = url;
	
	// Don't start recording this page again if it was just deleted
	if (url === thumbstrips_deletedUrl) {
		return;
	}
	
	thumbstrips_deletedUrl = "";
	
	thumbstrips_clearUpdaters();
	
	// Don't count about: or wyciwyg: pages or https pages
	if (url === "" || protocol === "about:" || protocol === "wyciwyg:" || protocol === "https:") {
		return ;
	}
	
	var domain = thumbstrips_trimDomain(host);
	
	// Add (and initialize) or update the node for this page
	//thumbstrips_currentNode = thumbstrips_filmstrip.addOrUpdateNode(url, url, domain, thumbstrips_initThumbnail);
	thumbstrips_currentNode = thumbstrips_filmstrip.addOrUpdateNode(url, url, domain, thumbstrips_init);
	
	thumbstrips_drawFilmstrip();
	
	thumbstrips_setUpdaters();
//	window.setTimeout(thumbstrips_setUpdaters, 50);
}

/*function thumbstrips_resetUpdaters(url, domain) {
	if (thumbstrips_recording === false) {
		return;
	}
	
//	thumbstrips_log("RESETTING, url = " + url + ", current = " + thumbstrips_currentUrl);
	
	// Loading a URL sometimes fires a location change event twice 
	if (url === thumbstrips_currentUrl) {
		return;
	}
	
	thumbstrips_currentUrl = url;
	
	// Don't start recording this page again if it was just deleted
	if (url === thumbstrips_deletedUrl) {
		return;
	}
	
	thumbstrips_deletedUrl = "";
	
	//This removes the updatePage timer, that's it
	thumbstrips_clearUpdaters();
	
	// Don't count about: pages
	if (url === "" || url.search("about:") === 0) {
		return ;
	}
	
	domain = thumbstrips_trimDomain(domain);
	
	// Add (and initialize) or update the node for this page
	thumbstrips_currentNode = thumbstrips_filmstrip.addOrUpdateNode(url, url, domain, thumbstrips_init);
	
	thumbstrips_drawFilmstrip();
	
	thumbstrips_setUpdaters();
}*/

// This removes the updatePage timer, that's it
function thumbstrips_clearUpdaters() {
	// A separate function because it's called from multiple places
	window.clearInterval(thumbstrips_pageUpdater);
}

function thumbstrips_setUpdaters() {
	// Set all updaters that rely on the page location (but fire them once before)
	// Commenting this out fixes the font problem
	//thumbstrips_updatePage();
	window.setTimeout(thumbstrips_updatePage, 500);
	thumbstrips_pageUpdater = window.setInterval(thumbstrips_updatePage, thumbstrips_prefs.getIntPref("time.page_update_frequency"));
}

function thumbstrips_capturePageText(node) {
	// recursive cases:
	
	// body, text, etc.: return textContent
	// frame, iframe: return textContent + f_cPT(frame.contentDocument.body)
	// frameset: return textContent + f_cPT(childNode[i]) for each i
	
	var text = "";
	
	// this should apply to HTMLBodyElement, Text, and others
	if (node.textContent) {
//		thumbstrips_log("TEXT");
//		thumbstrips_log("node = " + node);
//		thumbstrips_log("textContent = " + node.textContent);
		text = thumbstrips_removeComments(node.textContent);
	}
	
	if (node instanceof HTMLFrameElement || node instanceof HTMLIFrameElement) {
		// don't know if this check is necessary, but just to be on the safe side...
		if (node.contentDocument) {
			text += " " + thumbstrips_capturePageText(node.contentDocument.body ? node.contentDocument.body : node.contentDocument);
		}
	} else if (node instanceof HTMLFrameSetElement || node instanceof XMLDocument) {
		var nodes = node.childNodes;
		
		for (var i = 0; i < nodes.length; i++) {
			text += " " + thumbstrips_capturePageText(nodes.item(i));
		}
	}
	
	return thumbstrips_removeComments(text);
}

function thumbstrips_updatePage() {
	
	var node = thumbstrips_currentNode;
	
	if (!node) {
		return;
	}
	
	// Update the node (URL is already updated in resetUpdaters)
	
	node.title = content.document.title !== "" ? content.document.title : node.url;
	
	if (content.document) {
//		node.pageText = content.document.body.textContent;
		node.pageText = thumbstrips_capturePageText(content.document.body ? content.document.body : content.document);
	}
	
	node.updateCurrentVisit();
	
	// Capture the page
	
//	var wPage = browser.boxObject.width;
//	var hPage = browser.boxObject.height;
//	
//	var doc = browser.docShell.document;
//	var sLeft = 0;
//	var sTop = 0;
//	
//	try {
//		sLeft = doc.documentElement.scrollLeft;
//		sTop = doc.documentElement.scrollTop;	
//	
//		if (sLeft === 0 && doc.body !== null) {
//			sLeft = doc.body.scrollLeft;
//		}
//	
//		if (sTop === 0 && doc.body !== null) {
//			sTop = doc.body.scrollTop;
//		}
//	} catch (e) {
////		thumbstrips_log("scrolling not captured, url = " + node.url);
////		thumbstrips_log(e);
//	}

//	try {
//		wPage = browser.contentDocument.documentElement.clientWidth;
//		hPage = browser.contentDocument.documentElement.clientHeight;
//	
//		if (browser.contentDocument.body) {
//			if (!hPage || browser.contentDocument.body.offsetHeight !== browser.contentDocument.body.scrollHeight) {
//				hPage = browser.contentDocument.body.clientHeight;
//			}
//			if (!wPage || browser.contentDocument.body.offsetWidth !== browser.contentDocument.body.scrollWidth) {
//				wPage = browser.contentDocument.body.clientWidth;
//			}
//		}
//	} catch (e) {
////		frst_log("contentDocument.documentElement not valid, url = " + node.url);
////		thumbstrips_log(e);
//	}
//	
//	if (browser.boxObject) {
//		if (!wPage || wPage > browser.boxObject.width || wPage <= 0) {
//			wPage = browser.boxObject.width;
//		}
//
//		if (!hPage || hPage > browser.boxObject.height || hPage <= 0) {
//			hPage = browser.boxObject.height;
//		}
//	} else if (!wPage || !hPage || wPage <= 0 || hPage <= 0) {
//		thumbstrips_log("width/height failed to initialize, url = " + node.url);
//		return;
//	}
	
//	try {
//		sLeft = doc.documentElement.scrollLeft;
//		sTop = doc.documentElement.scrollTop;
//	} catch (e) {
////		thumbstrips_log("docShell.document.documentElement not valid, url = " + node.url);
////		thumbstrips_log(e);
//	}
//	
//	try {
//		if (sLeft === 0 && doc.body) {
//			sLeft = doc.body.scrollLeft;
//		}
//	
//		if (sTop === 0 && doc.body) {
//			sTop = doc.body.scrollTop;
//		}
//	} catch (e) {
////		thumbstrips_log("docShell.document.body not valid, url = " + node.url);
////		thumbstrips_log(e);
//	}

	try {
	
		var canvas = node.thumbnail;
		var context = canvas.getContext("2d");
		var browser = document.getElementById("content");
	
		var wPage = content.innerWidth - 16;
		var hPage = content.innerHeight - 16;
		
		var doc = browser.docShell.document;
		var sLeft = content.pageXOffset;
		var sTop = content.pageYOffset;
		
		if (!thumbstrips_tray.getAttribute("hidden")) {
			var end = content.innerHeight + content.scrollMaxY;
			var trayHeight = thumbstrips_tray.boxObject.height;
			
			if (end < (hPage + trayHeight)) {
				end = hPage + trayHeight;
			}
			
			var sMax = end - hPage - trayHeight;
			
			hPage += trayHeight;
				
			if (sTop > sMax) {
				sTop = sMax;
			}
		}
		
//		throw new Error("empty thumbnail test");
		
		var scale = canvas.height / hPage;
		var oldWidth = canvas.width;
		
		canvas.width = wPage * scale;
		canvas.displayWidth = canvas.displayHeight * canvas.width / canvas.height;
		
		canvas.style.width = canvas.displayWidth + "px";
		canvas.style.minWidth = canvas.style.width;
		canvas.style.maxWidth = canvas.style.width;
		
		context.save();
		context.scale(scale, scale);
		
		node.scrollX = sLeft;
		node.scrollY = sTop;
		context.drawWindow(content, sLeft, sTop, wPage, hPage, "rgba(255,255,255,1)");
		
		context.restore();
		
	} catch (e) {
		
		thumbstrips_log("Unable to thumbnail this page, url = " + node.url);
		thumbstrips_log(e);
		
		thumbstrips_copyImage(node.id, thumbstrips_emptyImage, false);
		
	}
	
	// After the width is set, unhide the canvas
	//if (canvas.getAttribute("hidden") === "true" && canvas.displayWidth > 0) {
	
	if (canvas && canvas.getAttribute("hidden") === "true" && canvas.displayWidth > 0) {
		canvas.removeAttribute("hidden");
//		canvas.parentNode.removeAttribute("hidden");
		thumbstrips_updateHiddenStatus(canvas.parentNode, "load", false);
		
		// if the canvas is visible and the tray is open, scroll to the new thumbnail
		/*if (!canvas.parentNode.getAttribute("hidden") && !thumbstrips_tray.getAttribute("hidden")) {
			thumbstrips_scrollbox.ensureElementIsVisible(canvas.parentNode);
		}*/
		//the above commented code has been moved to:
		thumbstrips_skipToCurrent();
	}
	
	// TODO If the width changed (significantly), adjust the scrollbar
	
//	if (Math.abs(canvas.width - oldWidth) > 0) {
//		var inner = document.getAnonymousNodes(thumbstrips_box).item(0);
//		var small = thumbstrips_box.boxObject.height;
//		var big = small + 16;
//		
//		if (thumbstrips_box.boxObject.width >= inner.boxObject.width) {
//			// no scrollbar necessary
//			thumbstrips_box.setAttribute("style", "height: " + small + "px;");
//			thumbstrips_hidden.removeAttribute("hidden");
//		} else {
//			// scrollbar necessary
//			thumbstrips_hidden.setAttribute("hidden", "true");
//			thumbstrips_box.setAttribute("style", "height: " + big + "px; overflow: auto;");
//			
//			// ensure current node is visible
//			thumbstrips_scrollbox.ensureElementIsVisible(document.getElementById(node.id));
//			thumbstrips_box.ensureElementIsVisible(document.getElementById(node.id));
//		}
//	}
	
	// Update the title and time on the filmstrip
	
	var titleLabel = document.getElementById(thumbstrips_titleClass + node.id);
	
	if (titleLabel) {
		titleLabel.setAttribute("value", node.title);
	}
	
}