// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// The Initial Developer of the Original Code is Aseem Kishore.                       //
// Portions created by Scott Williamson are Copyright (c) 2006.  All Rights Reserved. //
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

// -- PRINT PREVIEW BUG FIX -- //

// sort of a hack, but I think the hack is necessary because the bug is on a
// native Firefox level

// basically, if the filmstrip is recording, print preview causes a crash
// so I overrode the enterPrintPreview function to first stop recording
// and then start recording again when exiting print preview

// As a side note, this is also okay because of the way Firefox handles
// print preview -- you can't continue browsing until you exit print preview.

// for future reference, the functions were derived in the following manner:
// 1. open up the DOM inspector in firefox (ctrl+shift+I, or from tools menu)
// 2. inspect a Firefox window
// 3. navigate to the file menu -> print preview menu item
// 4. note the "command" id, cmd_printPreview or something similar
// 5. search the DOM for the element with that id, retrieve the command
// 6. see the "oncommand" attribute for that command, PrintUtils.printPreview
// 7. if you open up the javascript shell, you can do props(PrintUtils) to
//    see similar functions, like enterPrintPreview and exitPrintPreview

PrintUtils.tempEnterPrintPreview = PrintUtils.enterPrintPreview;
PrintUtils.tempExitPrintPreview = PrintUtils.exitPrintPreview;

PrintUtils.enterPrintPreview = function() {
	thumbstrips_wasRecording = thumbstrips_recording;
	thumbstrips_printPreviewing = true;
	thumbstrips_wasVisible = ! thumbstrips_tray.getAttribute("hidden");
	
	if (!thumbstrips_tray.getAttribute("hidden")) { // if tray is open
		thumbstrips_openTray();
	}
	
	if (thumbstrips_recording === true) {
		thumbstrips_toggleRecording();
	}
	
	PrintUtils.tempEnterPrintPreview();
}

PrintUtils.exitPrintPreview = function() {
	PrintUtils.tempExitPrintPreview();
	thumbstrips_printPreviewing = false;
	
	if (thumbstrips_wasRecording === true && thumbstrips_recording === false) {
		thumbstrips_toggleRecording();
	}
	
	if (thumbstrips_wasVisible === true && thumbstrips_tray.getAttribute("hidden")) {
		thumbstrips_openTray();
	}
	
	// when the user goes to file > print preview, the window loses focus for
	// a second, which means tempStopRecording is called. so when PrintUtils.
	// enterPrintPreview is called, recording is false. when we exit print
	// preview, recording stays false, but the window doesn't explicitly
	// regain focus, so we don't start recording. to correct this, we need to
	// call tempStartRecording, and we do that through the ppInterrupted flag.
	// this is very much a hack, unfortunately.
	
	if (thumbstrips_ppInterrupted === true) {
		thumbstrips_ppInterrupted = false;
		thumbstrips_tempStartRecording();
	}
}