// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// The Initial Developer of the Original Code is Aseem Kishore.                       //
// Portions created by Scott Williamson are Copyright (c) 2006.  All Rights Reserved. //
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

// -- SAVE DIALOG -- //


var thumbstrips_saveDialog;
var thumbstrips_stringBundle;

var thumbstrips_fileExtension = window.arguments[0];
var thumbstrips_filePath = window.arguments[1];
var thumbstrips_filterOn = window.arguments[2];
var thumbstrips_saveAcceptFunc = window.arguments[3];
var thumbstrips_saveCancelFunc = window.arguments[4];
 

function thumbstrips_onSaveDialogLoad() {
	thumbstrips_saveDialog = document.getElementById("thumbstrips_saveDialog");
	thumbstrips_stringBundle = document.getElementById("thumbstrips_saveDialogStringBundle");
	
	var filterStatus = document.getElementById("thumbstrips_saveFilterStatus");
	var filterButtons = document.getElementById("thumbstrips_saveFilterRadioGroup");
	
	if (thumbstrips_filterOn === true) {
		// keep the things enabled, just set the description
		filterStatus.setAttribute("value", thumbstrips_stringBundle.getString("saveFilterStatusFiltered"));
	} else { // nothing's being filtered
		// disable the buttons, set the description
		filterStatus.setAttribute("value", thumbstrips_stringBundle.getString("saveFilterStatusNone"));
		filterButtons.setAttribute("disabled", "true");
		
//		for (var i = 0; i < filterButtons.childNodes.length; i++) {
//			filterButtons.childNodes[i].setAttribute("disabled", "true");
//		}

		var shown = document.getElementById("thumbstrips_saveShown");
		var all = document.getElementById("thumbstrips_saveAll");
		
		shown.setAttribute("disabled", "true");
		all.setAttribute("disabled", "true");
	}
}


function thumbstrips_showShareFields() {
	var shareBox    = document.getElementById("thumbstrips_shareCheck").checked;
	var shareFields = document.getElementById("thumbstrips_shareFields"); 
	
	// Toggle display for sharing fields when checkbox is checked
	if(shareBox == true) {
		shareFields.setAttribute("collapsed", "true");
		window.sizeToContent();
	}
	else {
 		shareFields.setAttribute("collapsed", "false");
 		window.sizeToContent();
	}
}


function thumbstrips_saveFocusOnDisk() {
	var filmstripsNameBox = document.getElementById("thumbstrips_saveToFilmstripsNameBox");
	var diskNameBox = document.getElementById("thumbstrips_saveToDiskNameBox");
	var diskBrowse = document.getElementById("thumbstrips_saveToDiskBrowse");
	
	filmstripsNameBox.setAttribute("disabled", "true");
	diskNameBox.removeAttribute("disabled");
	diskBrowse.removeAttribute("disabled");
}

function thumbstrips_saveFocusOnFilmstrips() {
	var filmstripsNameBox = document.getElementById("thumbstrips_saveToFilmstripsNameBox");
	var diskNameBox = document.getElementById("thumbstrips_saveToDiskNameBox");
	var diskBrowse = document.getElementById("thumbstrips_saveToDiskBrowse");
	
	filmstripsNameBox.removeAttribute("disabled");
	diskNameBox.setAttribute("disabled", "true");
	diskBrowse.setAttribute("disabled", "true");
}


function thumbstrips_saveDiskBrowse() {
	
	var saveName = document.getElementById("thumbstrips_saveToFilmstripsNameBox").value;
	var fileBox = document.getElementById("thumbstrips_saveToDiskNameBox");
	
	var nsIFilePicker = Components.interfaces.nsIFilePicker;
	var fp = Components.classes["@mozilla.org/filepicker;1"]
				.createInstance(nsIFilePicker);
	
	fp.init(window, thumbstrips_stringBundle.getString("saveFilePickerName"), nsIFilePicker.modeSave);
	
	fp.appendFilter(thumbstrips_stringBundle.getString("saveFilePickerFilter") + " (*" + thumbstrips_fileExtension + ")", "*" + thumbstrips_fileExtension)
	fp.appendFilters(nsIFilePicker.filterAll);
	
	fp.defaultExtension = thumbstrips_fileExtension;
	fp.defaultString = (saveName === "") ? "" : (saveName + thumbstrips_fileExtension);

	var rv = fp.show();
	
	if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
		//fileBox.setAttribute("value", fp.file.path);
		// This fixes the save bug where the Ok button won't submit after an error with full path
		fileBox.value = fp.file.path;
		
	}
	
}


function thumbstrips_showSave() {
	var thumbstrips_shareCheckBox = document.getElementById("thumbstrips_shareCheck").checked;
	var friendCheck = document.getElementById("thumbstrips_shareFriendsCheck").checked;
	
	if(thumbstrips_shareCheckBox == true) {
		var answer = confirm("Your ThumbStrip will be shared on a public web site.  Is this ok?")
		if (answer){
			window.open("chrome://thumbstrips/content/pleasewaitdialog.xul", "", "chrome,modal,centerscreen");
	    	thumbstrips_onSaveDialogAccept();	
		}
		else {
			return false;
		}	
	}
	else {
		window.open("chrome://thumbstrips/content/pleasewaitdialog.xul", "", "chrome,modal,centerscreen");
	    thumbstrips_onSaveDialogAccept();	
	}	
}


function thumbstrips_onSaveDialogAccept() {
    	
    var fileName = "";
	var file = Components.classes["@mozilla.org/file/local;1"]
				.createInstance(Components.interfaces.nsILocalFile);
    
	var saveToDisk = document.getElementById("thumbstrips_saveToDisk").selected;
	
	if (saveToDisk === true) {
		
		fileName = document.getElementById("thumbstrips_saveToDiskNameBox").value;
		//file.initWithPath(fileName);
		    
		if (!file.exists()) {
			
			// fileName must be a full path in order to use initWidthPath
			try {
				file.initWithPath(fileName);
			} catch (e) {
				thumbstrips_log("Save dialog error: Full path name to file wasn't provided");
				thumbstrips_log(e);
				alert(thumbstrips_stringBundle.getString("errorFullPathRequired"));
				throw e;
			}
			
			try {
				file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
			} catch (e) {
				thumbstrips_log(e);
				alert(thumbstrips_stringBundle.getString("errorInvalidFileName"));
				return;
			}
		}
	} else {
		fileName = document.getElementById("thumbstrips_saveToFilmstripsNameBox").value;
		
		try {
			file = thumbstrips_getUniqueFile(fileName);
		} catch (e) {
			thumbstrips_log(e);
			alert(thumbstrips_stringBundle.getString("errorInvalidFileName"));
			return;
		}
	}
	
	var saveAll = document.getElementById("thumbstrips_saveAll").selected;
	
	//alert("thumbstrips_saveAcceptFunc: " + thumbstrips_saveAcceptFunc);
	
	// Store sharing information from save dialog
	var thumbstrips_shareCheckBox = document.getElementById("thumbstrips_shareCheck").checked;
	var nameBox     = document.getElementById("thumbstrips_shareName").value; 
	var descBox     = document.getElementById("thumbstrips_shareDesc").value; 
	var tagsBox     = document.getElementById("thumbstrips_shareTags").value; 
	var friendCheck = document.getElementById("thumbstrips_shareFriendsCheck").checked;
	
	if(thumbstrips_shareCheckBox == true) {
		if(nameBox == "") {
			nameBox = "anonymous";
		}
	}
	
	thumbstrips_saveAcceptFunc(file, saveAll, thumbstrips_shareCheckBox, nameBox, descBox, tagsBox, friendCheck);
}


function thumbstrips_onSaveDialogCancel() {
	thumbstrips_saveCancelFunc();
}


function thumbstrips_getUniqueFile(suggested) {
	var dir = Components.classes["@mozilla.org/file/directory_service;1"]
				.getService(Components.interfaces.nsIProperties)
				.get("UChrm", Components.interfaces.nsIFile);
	
	dir.append(thumbstrips_filePath);
	
	if (!dir.exists()) {
		dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0664);
	}
	
	if (!suggested) {
		suggested = "filmstrip" + thumbstrips_fileExtension;
//	} else if (suggested.substr(suggested.length - thumbstrips_fileExtension.length, thumbstrips_fileExtension.length) !== thumbstrips_fileExtension) {
	} else {
		suggested += thumbstrips_fileExtension;
	}
	
	var file = dir;

	file.append(suggested);
	file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
	
	return file;
}