// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// The Initial Developer of the Original Code is Aseem Kishore.                       //
// Portions created by Scott Williamson are Copyright (c) 2006.  All Rights Reserved. //
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

// -- SCROLLING -- //

function thumbstrips_mouseScroll(mouseX, mouseY) {
	if (thumbstrips_tray.getAttribute("hidden")) {
		return;
	} else if (thumbstrips_mousePause === true) {
		return;
	}
	
	// don't scroll if the menu is open
	
	//var menuButton = document.getElementById("thumbstrips_menuButton");
	var menuButton = document.getElementById("thumbstrips_menuButton");
	var contentButton = document.getElementById("thumbstrips_contentButton");
	
	if (menuButton.open === true || contentButton.open === true) {
		return;
	}
	
	if (!mouseX || !mouseY) {
		if (!thumbstrips_mouseX || !thumbstrips_mouseY) {
			return;
		}
		
		mouseX = thumbstrips_mouseX;
		mouseY = thumbstrips_mouseY;
	}
	
	var boxR = thumbstrips_box.boxObject.width / 2;
	var boxX = thumbstrips_box.boxObject.screenX + boxR;
	
	if (mouseY < thumbstrips_box.boxObject.screenY) {
		return;
//	} else if (mouseY > thumbstrips_box.boxObject.screenY + thumbstrips_box.boxObject.height) {
//		return;
	}
	
	// is the scrollbar visible? if so, don't scroll if mouse is on it
	// (note that when visible, thumbstrips_box is full width including scrollbar,
	// so we need to subtract 16px. when hidden, thumbstrips_box is 16px shorter,
	// so we need to add 16px for the "hidden" component.)
	
	if (thumbstrips_hidden.getAttribute("hidden") == "true") {	// scrollbar visible
		if (mouseY > thumbstrips_box.boxObject.screenY + thumbstrips_box.boxObject.height - 16) {
			return;
		}
	} else {	// scrollbar hidden
		if (mouseY > thumbstrips_box.boxObject.screenY + thumbstrips_box.boxObject.height + 16) {
			return;
		}
	}
	
	var rPlateau = thumbstrips_prefs.getIntPref("scrolling.plateau_radius");
	var maxSpeed = thumbstrips_prefs.getIntPref("scrolling.max_speed");
	
	var xLeftOuter = boxX - boxR;
	var xLeftInner = boxX - rPlateau;
	var xRightInner = boxX + rPlateau;
	var xRightOuter = boxX + boxR;
	
	if ((mouseX > xLeftInner) && (mouseX < xRightInner)) {
		// in plateau zone, no scrolling
		return;
	} else if ((mouseX < xLeftOuter - 1) || (mouseX > xRightOuter + 1)) {
		// outside filmstrip box, no scrolling
		return;
	}
	
	// METHOD 1: LINEAR
	
//	// in scroll zone, model it as a line mx + b, where m is rise/run
//	var rise = maxSpeed;
//	var run = xRightOuter - xRightInner;
//	var m = rise / run;
//	
//	// BUG FIX ... x can't be absolute position because then the right side
//	// scrolls slower than the left side, since the line is "off center"
//	// so... x is actually the difference between mouseX and boxX
//	
//	// calculate b knowing that x = xRightInner - boxX -> y = 0
//	var b = -m * (xRightInner - boxX);
//	var scrollSpeed = m * Math.abs(mouseX - boxX) + b;

	// METHOD 2: EXPONENTIAL
	
	// in scroll zone, model it as a curve e^(m*x) - 1
	// note that x is actually |mouseX-boxX| (see bug fix above)
	// we know that at mouseX = xRightOuter, m*x = n, y = e^n - 1 = maxSpeed
	// also, that at mouseX = xRightInner, m*x = 0, since y = e^0 - 1 = 0
	var n = Math.log(maxSpeed + 1);
	var m = n / (xRightOuter - boxX);
	
	var scrollSpeed = Math.exp(m * Math.abs(mouseX - boxX));
	
	scrollSpeed *= (mouseX < boxX) ? -1 : 1;
	
	thumbstrips_scrollbox.scrollBy(new Number(scrollSpeed), new Number(0));
}


// Mac fix: formerly just a call to thumbstrips_box.ensureElementIsVisible(node),
// but I think that (native) function sometimes throws an exception...
function thumbstrips_skipToCurrent() {
	//alert("skip to current");
	// on some computers, this function was crashing sometimes
	// (a try-catch block didn't help, it wasn't an unhandled exception)
	// so trying a timer, hopefully that fixes it -- IT DOES!
	// this is a workaround. This bug was happening exactly when the number
	// of filmstrips caused the thumbstrips_box to overflow (i.e. their combined
	// widths as greater than the width of the window) -- at this point, I
	// have a listener which adds a scrollbar (see drawing.js addScrollbar)
	// and I think this was creating some conflict. This timer fixes that.
	window.setTimeout(thumbstrips_skipToCurrentHelper, 50);
}


function thumbstrips_skipToCurrentHelper() {
    //alert("skipping to current helper");
		//thumbstrips_log("BEGIN skipping to current");
	
	// make sure the filmstrip is open first
	if (thumbstrips_tray.getAttribute("hidden") === true) {
		return;
	}
			
	//thumbstrips_log("milestone 1 in skipping to current");
	
	// if the current thumbnail is visible, scroll to the new thumbnail
	// (we don't have to worry about id + visitCount, because in every
	// sort, the current thumbnail's container has id = node.id
	if (thumbstrips_currentNode && thumbstrips_currentNode.id) {
		
		//thumbstrips_log("milestone 2 in skipping to current");
		
		var container = document.getElementById(thumbstrips_currentNode.id);
		
		//thumbstrips_log("milestone 3 in skipping to current");
	
		if (container && !container.getAttribute("hidden")) {
			
			//thumbstrips_log("milestone 4 in skipping to current");
			
			try {
				thumbstrips_scrollbox.ensureElementIsVisible(container);
			} catch (e) {
				thumbstrips_log("error scrolling to current node:");
				thumbstrips_log(e);
			}
		}
	}
	
	//thumbstrips_log("END skipping to current");
}



function thumbstrips_buttonScroll() {
	if (thumbstrips_scrollSpeed === 0) {
		alert("error, scroll speed = 0");
	}
	
	var sb = document.getElementById("statusbar-display");
	
	if (thumbstrips_scrollSpeed < 0) {
		sb.setAttribute("label", thumbstrips_rewindingStatus + " " + thumbstrips_scrollSpeed + "x...");
	} else {
		sb.setAttribute("label", thumbstrips_fforwardingStatus + " " + thumbstrips_scrollSpeed + "x...");
	}
	
	thumbstrips_scrollbox.scrollBy(new Number(thumbstrips_scrollSpeed), new Number(0));
	
	var inner = document.getAnonymousNodes(thumbstrips_box).item(0);
	
	// if rewinding and hit the left edge, pause
	if (thumbstrips_scrollSpeed < 0 &&
			(inner.boxObject.screenX + 1) >= thumbstrips_box.boxObject.screenX) {
		thumbstrips_pause();
	}
	
	// if fforwarding and hit the right edge, pause
	if (thumbstrips_scrollSpeed > 0 &&
			(inner.boxObject.screenX + inner.boxObject.width - 1) <=
			(thumbstrips_box.boxObject.screenX + thumbstrips_box.boxObject.width)) {
		thumbstrips_pause();
	}
}


function thumbstrips_skipBack() {
	thumbstrips_pause();
	thumbstrips_scrollbox.scrollTo(0, 0);
}


function thumbstrips_skipForward() {
	thumbstrips_pause();
	var inner = document.getAnonymousNodes(thumbstrips_box).item(0);
	thumbstrips_scrollbox.scrollTo(inner.boxObject.width, 0);
}


function thumbstrips_pause() {
	window.clearInterval(thumbstrips_buttonScrollUpdater);
	
	if (thumbstrips_scrollSpeed && thumbstrips_scrollSpeed !== 0) {
		thumbstrips_scrollSpeed = 0;
		document.getElementById("statusbar-display").setAttribute("label", thumbstrips_pausedStatus + "...");
	}
}


function thumbstrips_rewind() {
	var minSpeed = thumbstrips_prefs.getIntPref("scrolling.min_speed");
	var maxSpeed = thumbstrips_prefs.getIntPref("scrolling.max_speed");
	var multiple = thumbstrips_prefs.getIntPref("scrolling.multiple");
	
	if (!thumbstrips_scrollSpeed || thumbstrips_scrollSpeed === 0) {	// we were paused
		thumbstrips_scrollSpeed = -1 * minSpeed;
		thumbstrips_buttonScrollUpdater = window.setInterval(thumbstrips_buttonScroll, thumbstrips_prefs.getIntPref("time.button_scroll_frequency"));
	} else if (thumbstrips_scrollSpeed < 0) {	// we were already rewinding
		var newSpeed = thumbstrips_scrollSpeed * multiple;
		if (newSpeed < -1 * maxSpeed) {	// reset if we are at max speed
			newSpeed = -1 * minSpeed;
		}
		
		thumbstrips_scrollSpeed = newSpeed;
	} else {	// we were fast forwarding
		thumbstrips_scrollSpeed = -1 * minSpeed;
	}
}


function thumbstrips_fforward() {
	var minSpeed = thumbstrips_prefs.getIntPref("scrolling.min_speed");
	var maxSpeed = thumbstrips_prefs.getIntPref("scrolling.max_speed");
	var multiple = thumbstrips_prefs.getIntPref("scrolling.multiple");
	
	if (!thumbstrips_scrollSpeed || thumbstrips_scrollSpeed === 0) {	// we were paused
		thumbstrips_scrollSpeed = minSpeed;
		thumbstrips_buttonScrollUpdater = window.setInterval(thumbstrips_buttonScroll, thumbstrips_prefs.getIntPref("time.button_scroll_frequency"));
	} else if (thumbstrips_scrollSpeed > 0) {	// we were already fforwarding
		var newSpeed = thumbstrips_scrollSpeed * multiple;
		
		if (newSpeed > maxSpeed) {	// reset if we are at max speed
			newSpeed = minSpeed;
		}
		
		thumbstrips_scrollSpeed = newSpeed;
	} else {	// we were rewinding
		thumbstrips_scrollSpeed = minSpeed;
	}
}


// new stuff
function thumbstrips_tempStopMouseScroll() {
	if (thumbstrips_prefs.getBoolPref("options.mouse_scroll") === true) {
		window.removeEventListener("mousemove", thumbstrips_updateMousePosition, true);
		window.clearInterval(thumbstrips_mouseScrollUpdater);
		window.addEventListener("focus", thumbstrips_tempStartMouseScroll, true);
		thumbstrips_mousePause = true;
	}
}

// new stuff
function thumbstrips_tempStartMouseScroll() {
	if (thumbstrips_prefs.getBoolPref("options.mouse_scroll") === true && thumbstrips_mousePause === true) {
		window.addEventListener("mousemove", thumbstrips_updateMousePosition, true);
		thumbstrips_mouseScrollUpdater = window.setInterval(thumbstrips_mouseScroll, thumbstrips_prefs.getIntPref("time.mouse_update_frequency"));
		window.removeEventListener("focus", thumbstrips_tempStartMouseScroll, true);
		thumbstrips_mousePause = false;
	}
}


function thumbstrips_toggleMouseScrollPreference() {
	thumbstrips_togglePreference("options.mouse_scroll");
}


function thumbstrips_setMouseScroll(value) {
	window.removeEventListener("mousemove", thumbstrips_updateMousePosition, true);
	window.clearInterval(thumbstrips_mouseScrollUpdater);
		
	if (value === true) {
//		alert("scrolling turned on");
		window.addEventListener("mousemove", thumbstrips_updateMousePosition, true);
		thumbstrips_mouseScrollUpdater = window.setInterval(thumbstrips_mouseScroll, thumbstrips_prefs.getIntPref("time.mouse_update_frequency"));
	} else {
//		alert("scrolling turned off");
	}
	
	var item = document.getElementById("thumbstrips_mouseScrollMenuItem");
	item.setAttribute("checked", value);
}