// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// The Initial Developer of the Original Code is Aseem Kishore.                       //
// Portions created by Scott Williamson are Copyright (c) 2006.  All Rights Reserved. //
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

// webpagenode.js

var thumbstrips_WN_CHECK_REP = true;

function thumbstrips_WebpageNode(title, url, domain, thumbnail, pageText, id, scrollX, scrollY, aVisitList, aVisitCount, aVisitTimeInMilliseconds) {
	
	// Private fields
	
	var self = this;
	var visitList = aVisitList ? thumbstrips_copy(aVisitList) : [ new Date() ];
	var visitCount = aVisitCount ? parseInt(aVisitCount) : 1;
	var visitTimeInMilliseconds = aVisitTimeInMilliseconds ? parseInt(aVisitTimeInMilliseconds) : 0;
	
	// if aVisitList was supplied, convert the strings to Dates
	
	if (aVisitList) {
		for (var i = 0; i < visitList.length; i++) {
			visitList[i] = new Date(visitList[i]);
		}
	}
	
	
	// Public properties
	
	this.title = title;
	this.url = url;
	this.domain = domain;
	this.thumbnail = thumbnail;
	this.pageText = pageText;
	this.id = id;
	this.scrollX = scrollX;
	this.scrollY = scrollY;
	
	
	// Private helpers
	
	function checkRep() {
		if (thumbstrips_WN_CHECK_REP === false) {
			return;
		}
		
		var cr = self.toString() + " checkRep";
		
		// visitList is an array of dates
		thumbstrips_checkArrayType(cr, visitList);
		for (var i = 0; i < visitList.length; i++) {
			thumbstrips_checkDateType(cr, visitList[i]);
		}
		
		// visitCount is a number
		thumbstrips_checkType(cr, visitCount, "number");
		
		// visitCount >= 1
		thumbstrips_checkTrue(cr, visitCount >= 1, "visitCount < 1");
		
		// visitCount = length of visitList
		thumbstrips_checkTrue(cr, visitCount === visitList.length, "visitCount != visitList.length");
		
		// visitTimeInMilliseconds is a number
		thumbstrips_checkType(cr, visitTimeInMilliseconds, "number");
		
		// visitTimeInMilliseconds >= 0
		thumbstrips_checkTrue(cr, visitTimeInMilliseconds >= 0, "visitTimeInMilliseconds < 0");
	}
	
	
	// Public methods
	
	this.toString = function() {
		return self.title + " (" + self.url + ")";
	};
	
	this.getVisitList = function() {
		return thumbstrips_copy(visitList);
	};
	
	this.getVisitCount = function() {
		return visitCount;
	};
	
	this.getVisitTimeInMilliseconds = function() {
		return visitTimeInMilliseconds;
	};
	
	this.beginNewVisit = function() {
		visitList[visitCount] = new Date();
		visitCount++;
		
		checkRep();
	};
	
	this.updateCurrentVisit = function(dontAddTime) {
		var temp = new Date();
		
		if (!dontAddTime) {
			visitTimeInMilliseconds += (temp - visitList[visitCount - 1]);
		}
		
		visitList[visitCount - 1] = temp;
		
		checkRep();
	};
	
	
	checkRep();
	
}